# This file is a part of alpha.

import ambient

def _resolve_window(window):
	if window is None:
	    return ambient.active_window()
	return window


# splitting windows

def split_window(window = None, n = 0):
	'''Splits the window.'''
	return _resolve_window(window).split()

def split_window_side_by_side(window = None, n = 0):
	'''Splits the window side-by-side.'''
	return _resolve_window(window).split_side_by_side()


# deleting windows

def close_window(window = None, n = 0):
	'''Closes the window.'''
	_resolve_window(window).close()

def close_other_windows(window = None, n = 0):
	'''Closes the all windows except the active or the given.'''
	# todo: this code can't select the window wants to live.
	ambient.windows().unsplit_all()

def close_windows_on(buffer, n = 0):
	raise NotImplementedError()


# cyclic window ordering

def next_window(window = None, n = 0):
	'''Activates the next window.'''
#	_resolve_window(window).activate()
	ambient.windows().activate_next()
	return ambient.active_window()

def previous_window(window = None, n = 0):
	'''Activates the previous window.'''
#	_resolve_window(window).activate()
	ambient.windows().activate_previous()
	return ambient.active_window()


# displaying buffers

def switch_to_buffer(buffer):
	raise NotImplementedError()

def switch_to_buffer_other_window(buffer):
	raise NotImplementedError()

def replace_buffer_in_windows(buffer):
	raise NotImplementedError()


# choosing window

def display_buffer(buffer, not_this_window = False):
	raise NotImplementedError()


# vertical scrolling

def scroll_up(window = None, n = None):
	window = _resolve_window(window)
	if n is None:
		n = window.number_of_visible_lines
	_resolve_window(window).scroll(-n, 0)

def scroll_down(window = None, n = None):
	window = _resolve_window(window)
	if n is None:
		n = window.number_of_visible_lines
	_resolve_window(window).scroll(n, 0)

def scroll_other_window(n = None):
	raise NotImplementedError()

def recenter(window = None, n = None):
	raise NotImplementedError()


# horizontal scrolling

# todo: this command name has localization problem.
def scroll_left(window = None, n = None):
	window = _resolve_window(window)
	if n is None:
		n = window.number_of_visible_columns
	_resolve_window(window).scroll(0, -n)

# todo: this command name has localization problem.
def scroll_right(window = None, n = None):
	window = _resolve_window(window)
	if n is None:
		n = window.number_of_visible_columns
	_resolve_window(window).scroll(0, n)


# nonstandard

def _next_buffer(previous = False, n = 0):
	'''Shows the next/previous buffer in the active window.'''
	buffers = ambient.buffers()
	active_buffer = ambient.active_buffer()
	for i in range(0, len(buffers)):
		if buffers[i] is active_buffer:
			if not previous:
				i += 1
				if i == len(buffers):
					i = 0
			else:
				if i == 0:
					i = len(buffers) - 1
				else:
					i -= 1
			ambient.active_window().select(buffers[i])
			break

def next_buffer(n = 1):
	return _next_buffer(False, n)

def previous_buffer(n = 1):
	return _next_buffer(True, n)


from ambient.bindings import *
k = InputMappingScheme.get()
k.define(KeyStroke(NaturalKey.page_down, ModifierKey.ctrl), scroll_down)
k.define(KeyStroke(NaturalKey.down, ModifierKey.ctrl), lambda window = None, n = None : scroll_down(window, 1))
k.define(KeyStroke(NaturalKey.page_up, ModifierKey.ctrl), scroll_up)
k.define(KeyStroke(NaturalKey.up, ModifierKey.ctrl), lambda window = None, n = None : scroll_up(window, 1))
k.define(KeyStroke(NaturalKey.f6), next_window)
k.define(KeyStroke(NaturalKey.f6, ModifierKey.shift), previous_window)
k.define(KeyStroke(NaturalKey.tab, ModifierKey.ctrl), next_buffer)
k.define(KeyStroke(NaturalKey.tab, ModifierKey.ctrl_shift), previous_buffer)
